"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const bn_js_1 = require("bn.js");
const core_1 = require("@oclif/core");
const base_1 = tslib_1.__importDefault(require("../base"));
const token_1 = require("@tonic-foundation/token");
class ListMarkets extends base_1.default {
    async getRow(marketId, index) {
        const market = await this.tonic.getMarket(marketId);
        const [baseToken, quoteToken] = await Promise.all([
            (0, token_1.ftOrNativeNearMetadata)(this.account, market.baseTokenId),
            (0, token_1.ftOrNativeNearMetadata)(this.account, market.quoteTokenId),
        ]);
        const baseSymbol = baseToken.symbol;
        const quoteSymbol = quoteToken.symbol;
        return {
            index,
            id: market.id,
            pair: `${baseSymbol}/${quoteSymbol}`,
            baseSymbol,
            quoteSymbol,
            baseToken: market.baseTokenId,
            quoteToken: market.quoteTokenId,
            baseTick: market.quantityBnToNumber(new bn_js_1.BN(market.baseLotSize)),
            quoteTick: market.priceBnToNumber(new bn_js_1.BN(market.quoteLotSize)),
        };
    }
    async run() {
        const { flags } = await this.parse(ListMarkets);
        const allMarkets = await this.tonic.listMarkets(flags.offset, flags.limit);
        const markets = flags.includeInactive
            ? allMarkets
            : // TODO: bump SDK version to remove this
                // @ts-ignore
                allMarkets.filter((m) => m.state === 'Active');
        const rows = await Promise.all(markets.map((m, idx) => this.getRow(m.id, flags.offset + idx)));
        core_1.CliUx.Table.table(rows, {
            index: {
                header: 'Index',
            },
            id: {
                header: 'ID',
            },
            pair: {
                header: 'Pair',
            },
            baseTick: {
                header: `Quantity min tick`,
                get: ({ baseTick, baseSymbol }) => {
                    return `${baseTick} ${baseSymbol}`;
                },
            },
            quoteTick: {
                header: `Price min tick`,
                get: ({ quoteTick, quoteSymbol }) => {
                    return `${quoteTick} ${quoteSymbol}`;
                },
            },
        });
    }
}
exports.default = ListMarkets;
ListMarkets.description = `List markets in the DEX contract`;
ListMarkets.examples = [`$ tonic list-markets`];
ListMarkets.flags = {
    ...base_1.default.flags,
    offset: core_1.Flags.integer({
        description: 'Market ID offset',
        default: 0,
    }),
    limit: core_1.Flags.integer({
        description: 'Limit on number of markets returned',
        default: 100,
    }),
    includeInactive: core_1.Flags.boolean({
        description: 'Include inactive markets',
        default: false,
    }),
};
ListMarkets.args = [];
