"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrderTypeFlag = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const base_1 = tslib_1.__importStar(require("../base"));
exports.OrderTypeFlag = core_1.Flags.build({
    parse: async (s) => {
        if (s.toUpperCase() === 'LIMIT') {
            return 'Limit';
        }
        if (s.toUpperCase() === 'MARKET') {
            return 'Market';
        }
        if (s.toUpperCase() === 'POSTONLY') {
            return 'PostOnly';
        }
        throw new Error('Invalid order type');
    },
});
class PlaceOrder extends base_1.default {
    async run() {
        const { args, flags } = await this.parse(PlaceOrder);
        if (flags.type === 'market') {
            throw new Error('Market order not supported yet');
        }
        if (!flags.buy && !flags.sell) {
            throw new Error('Missing order direction (pass --buy or --sell)');
        }
        const { market: marketId } = args;
        const market = await this.tonic.getMarket(marketId);
        this.log(`Sending transaction...`);
        const side = flags.buy ? 'Buy' : 'Sell';
        const { response, executionOutcome } = await market.placeOrder({
            limitPrice: flags.price,
            quantity: flags.quantity,
            orderType: flags.orderType,
            side,
        });
        this.logTransaction(executionOutcome);
        this.log(`Order ${response.id} placed`);
        // whatever
        console.log(Object.fromEntries(Object.keys(response).map((k) => {
            return [k, response[k].toString()];
        })));
    }
}
exports.default = PlaceOrder;
PlaceOrder.description = 'Place an order.';
PlaceOrder.examples = [
    `$ tonic place-order NEAR-ft:ft.examples.testnet \
    --direction buy \
    --price 100 \
    --post-only`,
];
PlaceOrder.flags = {
    ...base_1.default.flags,
    type: core_1.Flags.enum({
        options: ['limit', 'market'],
        char: 't',
        description: 'Order type',
        default: 'limit',
        required: true,
        parse: (s) => {
            if (s === 'limit') {
                return s; // oclif type issue
            }
            throw new Error(`${s} orders unimplemented`);
        },
    }),
    buy: core_1.Flags.boolean({
        description: 'Buy',
        exactlyOne: ['buy', 'sell'],
    }),
    sell: core_1.Flags.boolean({
        description: 'Sell',
        exactlyOne: ['buy', 'sell'],
    }),
    price: (0, base_1.FloatFlag)({
        char: 'p',
        description: 'Order price as a human-readable number, eg 1.23',
        required: true,
    }),
    quantity: (0, base_1.FloatFlag)({
        char: 'q',
        description: 'Order quantity as a human-readable number, eg 4.56',
        required: true,
    }),
    orderType: (0, exports.OrderTypeFlag)({
        char: 't',
        description: 'Order type (market, limit, postonly)',
        default: 'Limit',
    }),
    ioc: core_1.Flags.boolean({
        default: false,
        exclusive: ['post-only'],
    }),
};
PlaceOrder.args = [
    {
        name: 'market',
        description: 'Market ID',
        required: true,
    },
];
